import os
import socket
import ssl
import sys
import mysql.connector

import L2Dcfg
import parse_log_lines

# BEGINNING
if __name__ == "__main__":
    if len(sys.argv) > 1:   # Arguments were passed.
##        for i, argv in enumerate(sys.argv):
##            print(f"Argument {i}: {argv}")
        file_path = sys.argv[1]
        print(f"Processing: {file_path}")
    else:
        print("Apache Log Analysis and Database Entry Python Script")
        print("Catwhisperer - https://happycattech.com")
        print("Version: 1.01")
        print("")

        # Parse for arguments passed:
        # - file path. Use it if it exists.
        # - Pausing between lines flag, (-p)
        # - L2Dcfg.DEBUG output. (-d)

        file_path = ""  # Initialize positively null path string.
        while not file_path:
            file_path = input("Enter the path to the log file you wish to parse: ")
            if not file_path:
                print("Path can't be empty. Pleast try again.", "\n")
                continue

    # Attempt to connect to the database.
    ladb = mysql.connector.connect(
        # This should be put in a configuration file at some point.
        host = L2Dcfg.HOST,
        user = L2Dcfg.USER,
        password = L2Dcfg.PWD,
        database = L2Dcfg.DATABASE
    )
    if L2Dcfg.DEBUG: print(ladb)
    if os.path.exists(file_path):   # Check that the path exists. Maybe also check read perms.
        with open(file_path, 'r') as log_file:
            # Configure the cursor pointor.
            batch_data = []

            for line in log_file:
                result = parse_log_lines.process_line( line.strip() )
                if L2Dcfg.DEBUG:
                    if result: sys.exit()
                batch_data.append(result)
                
            if L2Dcfg.DEBUG: print("End of file reached.")

            if batch_data:
                db_cursor = ladb.cursor()
                if L2Dcfg.DEBUG_BUFFER:
                    for index, obj in enumerate(batch_data):
                        print(f"Line {index +1}:{obj}")
                        input("Press Enter to continue...")

                try:
                    sql = "INSERT INTO log_data (log_ipaddress, log_remlogname, log_remuser, log_timestamp, log_request, log_statuscode, log_bytessent, log_referer, log_useragent, log_forensicid) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
                    # Efficient bulk insert
                    db_cursor.executemany(sql, batch_data)
                    print(f"Inserted {len(batch_data)} log entries.")

                except Exception as e:
                    # Rollback on error
                    ladb.rollback()
                    print(f"Error: {e}")
                    if L2Dcfg.DEBUG_DB:
                        for index, obj in enumerate(batch_data):
                            print(f"Line {index +1}:{obj}")

                finally:
                    db_cursor.close()
                    # Commit after batch processing
                    ladb.commit()
                    ladb.close

            sys.exit()

    else:
        print({file_path},"is an invalid path.")
        print("Could not find the file pertaining to that path.", "\n")
        sys.exit()


